@@ -134,6 +134,7 @@ gboolean bacon_video_widget_seek		 (BaconVideoWidget *bvw,
 gboolean bacon_video_widget_seek_time		 (BaconVideoWidget *bvw,
 						  gint64 time,
 						  GError **error);
+gboolean bacon_video_widget_can_direct_seek	 (BaconVideoWidget *bvw);
 float bacon_video_widget_get_position            (BaconVideoWidget *bvw);
 gint64 bacon_video_widget_get_current_time       (BaconVideoWidget *bvw);
 gint64 bacon_video_widget_get_stream_length      (BaconVideoWidget *bvw);
@@ -194,7 +195,6 @@ typedef enum {
 	BVW_INFO_ALBUM,
 	BVW_INFO_DURATION,
 	BVW_INFO_TRACK_NUMBER,
-	BVW_INFO_CDINDEX,
 	/* Video */
 	BVW_INFO_HAS_VIDEO,
 	BVW_INFO_DIMENSION_X,
@@ -218,7 +218,8 @@ typedef enum {
 	VISUAL_SMALL,
 	VISUAL_NORMAL,
 	VISUAL_LARGE,
-	VISUAL_EXTRA_LARGE
+	VISUAL_EXTRA_LARGE,
+	NUM_VISUAL_QUALITIES
 } VisualsQuality;
 
 gboolean bacon_video_widget_set_show_visuals	  (BaconVideoWidget *bvw,
