@@ -152,6 +152,18 @@ totem_gdk_window_set_invisible_cursor (GdkWindow *window)
 	g_object_unref (empty_bitmap);
 }
 
+void
+totem_gdk_window_set_waiting_cursor (GdkWindow *window)
+{
+	GdkCursor *cursor;
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (window, cursor);
+	gdk_cursor_unref (cursor);
+
+	gdk_flush ();
+}
+
 void
 totem_create_symlinks (const char *orig, const char *dest)
 {
@@ -358,3 +370,26 @@ totem_ratio_fits_screen (GdkWindow *video_window, int video_width,
 
 	return TRUE;
 }
+
+char *
+totem_resolve_relative_link (const char *old_mrl, const char *new_mrl)
+{
+	char *ret, *tmp, *l;
+
+	g_return_val_if_fail (new_mrl != NULL, NULL);
+	g_return_val_if_fail (old_mrl != NULL, NULL);
+
+	/* if new mrl is already absolute, just return it as it is */
+	if (strstr (new_mrl, "://") != NULL)
+		return g_strdup (new_mrl);
+
+	tmp = g_strdup (old_mrl);
+	if ((l = strrchr (tmp, '/')))
+		*l = '\0';
+
+	ret = g_strconcat (tmp, "/", new_mrl, NULL);
+	g_free (tmp);
+
+	return ret;
+}
+
