@@ -47,6 +47,7 @@
 #define RECURSE_LEVEL_MAX 4
 #define DIR_MIME_TYPE "x-directory/normal"
 #define EXTINF "#EXTINF:"
+#define DEBUG(x) { if (parser->priv->debug != FALSE) x; }
 
 typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, const char *url, gpointer data);
 static gboolean totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, const char *url);
@@ -75,11 +76,13 @@ struct TotemPlParserPrivate
 	guint recurse_level;
 	gboolean fallback;
 	gboolean recurse;
+	gboolean debug;
 };
 
 enum {
 	PROP_NONE,
 	PROP_RECURSE,
+	PROP_DEBUG
 };
 
 /* Signals */
@@ -120,6 +123,14 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (object_class,
+					 PROP_DEBUG,
+					 g_param_spec_boolean ("debug",
+							       "debug",
+							       "Whether or not to enable debugging output", 
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	/* Signals */
 	totem_pl_parser_table_signals[ENTRY] =
 		g_signal_new ("entry",
@@ -160,7 +171,9 @@ totem_pl_parser_set_property (GObject *object,
 	case PROP_RECURSE:
 		parser->priv->recurse = g_value_get_boolean (value);
 		break;
-
+	case PROP_DEBUG:
+		parser->priv->debug = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -180,7 +193,9 @@ totem_pl_parser_get_property (GObject *object,
 	case PROP_RECURSE:
 		g_value_set_boolean (value, parser->priv->recurse);
 		break;
-
+	case PROP_DEBUG:
+		g_value_set_boolean (value, parser->priv->debug);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -204,7 +219,7 @@ totem_pl_parser_new (void)
 }
 
 static const char *
-my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
+my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlParser *parser)
 {
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
@@ -220,6 +235,7 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 	if (result != GNOME_VFS_OK) {
 		if (result == GNOME_VFS_ERROR_IS_DIRECTORY)
 			return DIR_MIME_TYPE;
+		DEBUG(g_print ("URL '%s' couldn't be opened in _get_mime_type_with_data: '%s'\n", uri, gnome_vfs_result_to_string (result)));
 		return NULL;
 	}
 
@@ -250,9 +266,14 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 	} while (result == GNOME_VFS_OK
 			&& total_bytes_read < MIME_READ_CHUNK_SIZE);
 
-	/* Close the file. */
-	result = gnome_vfs_close (handle);
+	/* Close the file but don't overwrite the possible error */
+	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF)
+		gnome_vfs_close (handle);
+	else
+		result = gnome_vfs_close (handle);
+
 	if (result != GNOME_VFS_OK) {
+		DEBUG(g_print ("URL '%s' couldn't be read or closed in _get_mime_type_with_data: '%s'\n", uri, gnome_vfs_result_to_string (result)));
 		g_free (buffer);
 		return NULL;
 	}
@@ -267,16 +288,6 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 	return mimetype;
 }
 
-static char*
-totem_pl_parser_set_method (const char *url, const char *method)
-{
-	char *start;
-
-	g_return_val_if_fail (strstr (url, "://"), NULL);
-	start = strstr (url, "://");
-	return g_strdup_printf ("%s%s", method, start);
-}
-
 static char*
 totem_pl_parser_base_url (const char *url)
 {
@@ -605,12 +616,19 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		}
 		g_free (title);
 
-		if (dos_compatible != FALSE)
+		if (dos_compatible == FALSE) {
+			char *tmp;
+			tmp = totem_pl_parser_relative (url, output);
+			if (tmp == NULL && g_str_has_prefix (url, "file:")) {
+				path2 = g_filename_from_uri (url, NULL, NULL);
+			} else {
+				path2 = tmp;
+			}
+		} else {
 			path2 = totem_pl_parser_url_to_dos (url, output);
-		else
-			path2 = totem_pl_parser_relative (url, output);
+		}
 
-		buf = g_strdup_printf ("%s%s", path2, cr);
+		buf = g_strdup_printf ("%s%s", path2 ? path2 : url, cr);
 		g_free (path2);
 		g_free (url);
 
@@ -674,10 +692,15 @@ read_ini_line_int (char **lines, const char *key)
 		return -1;
 
 	for (i = 0; (lines[i] != NULL && retval == -1); i++) {
-		if (g_ascii_strncasecmp (lines[i], key, strlen (key)) == 0) {
+		char *line = lines[i];
+
+		while (*line == '\t' || *line == ' ')
+			line++;
+
+		if (g_ascii_strncasecmp (line, key, strlen (key)) == 0) {
 			char **bits;
 
-			bits = g_strsplit (lines[i], "=", 2);
+			bits = g_strsplit (line, "=", 2);
 			if (bits[0] == NULL || bits [1] == NULL) {
 				g_strfreev (bits);
 				return -1;
@@ -701,11 +724,16 @@ read_ini_line_string (char **lines, const char *key, gboolean dos_mode)
 		return NULL;
 
 	for (i = 0; (lines[i] != NULL && retval == NULL); i++) {
-		if (g_ascii_strncasecmp (lines[i], key, strlen (key)) == 0) {
+		char *line = lines[i];
+
+		while (*line == '\t' || *line == ' ')
+			line++;
+
+		if (g_ascii_strncasecmp (line, key, strlen (key)) == 0) {
 			char **bits;
 			ssize_t len;
 
-			bits = g_strsplit (lines[i], "=", 2);
+			bits = g_strsplit (line, "=", 2);
 			if (bits[0] == NULL || bits [1] == NULL) {
 				g_strfreev (bits);
 				return NULL;
@@ -938,8 +966,10 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, c
 
 	if (lines[i] == NULL
 			|| g_ascii_strncasecmp (lines[i], "[playlist]",
-				(gsize)strlen ("[playlist]")) != 0)
+				(gsize)strlen ("[playlist]")) != 0) {
+		DEBUG (g_print ("Couldn't recognise URL '%s' as a .pls playlist\n", url));
 		goto bail;
+	}
 
 	playlist_title = read_ini_line_string (lines,
 			"X-GNOME-Title", dos_mode);
@@ -952,8 +982,10 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, c
 
 	/* numberofentries=? */
 	num_entries = read_ini_line_int (lines, "numberofentries");
-	if (num_entries == -1)
+	if (num_entries == -1) {
+		DEBUG (g_print ("Couldn't get the numberofentries for '%s'\n", url));
 		goto bail;
+	}
 
 	retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
@@ -1215,13 +1247,6 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 	g_free (url);
 
-	if (g_str_has_prefix (fullpath, "mms:") != FALSE) {
-		char *tmp;
-		tmp = totem_pl_parser_set_method (fullpath, "mmsh");
-		g_free (fullpath);
-		fullpath = tmp;
-	}
-
 	/* .asx files can contain references to other .asx files */
 	retval = totem_pl_parser_parse_internal (parser, fullpath);
 	if (retval != TOTEM_PL_PARSER_RESULT_SUCCESS) {
@@ -1461,6 +1486,69 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 	return totem_pl_parser_add_asf_parser (parser, url, data);
 }
 
+static TotemPlParserResult
+totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser, const char *url, gpointer data)
+{
+	xmlDocPtr doc;
+	xmlNodePtr node;
+	char *contents = NULL, *src;
+	int size;
+
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+
+	doc = xmlParseMemory (contents, size);
+	if (doc == NULL)
+		doc = xmlRecoverMemory (contents, size);
+	g_free (contents);
+
+	/* If the document has no root, or no name */
+	if(!doc || !doc->children
+			|| !doc->children->name
+			|| g_ascii_strcasecmp ((char *)doc->children->name,
+				"quicktime") != 0) {
+		if (doc != NULL)
+			xmlFreeDoc (doc);
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
+
+	if (strstr ((char *) doc->children->content, "type=\"application/x-quicktime-media-link\"") == NULL) {
+		xmlFreeDoc (doc);
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
+
+	node = doc->children->next;
+	if (!node || !node->name
+			|| g_ascii_strcasecmp ((char *) node->name,
+				"embed") != 0) {
+		xmlFreeDoc (doc);
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
+
+	src = (char *) xmlGetProp (node, (guchar *)"src");
+	if (!src) {
+		xmlFreeDoc (doc);
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
+
+	totem_pl_parser_add_one_url (parser, src, NULL);
+
+	xmlFreeDoc (doc);
+
+	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+}
+
+static TotemPlParserResult
+totem_pl_parser_add_quicktime (TotemPlParser *parser, const char *url, gpointer data)
+{
+	if (data == NULL || strstr (data, "<?quicktime") == NULL) {
+		totem_pl_parser_add_one_url (parser, url, NULL);
+		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
+	return totem_pl_parser_add_quicktime_metalink (parser, url, data);
+}
+
 static TotemPlParserResult
 totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer data)
 {
@@ -1650,6 +1738,7 @@ static PlaylistTypes dual_types[] = {
 	{ "text/plain", totem_pl_parser_add_ra },
 	{ "video/x-ms-asf", totem_pl_parser_add_asf },
 	{ "video/x-ms-wmv", totem_pl_parser_add_asf },
+	{ "video/quicktime", totem_pl_parser_add_quicktime },
 };
 
 static gboolean
@@ -1729,7 +1818,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	gpointer data = NULL;
 	TotemPlParserResult ret = TOTEM_PL_PARSER_RESULT_ERROR;
 	char *super;
-	gboolean found;
+	gboolean found = FALSE;
 
 	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -1737,6 +1826,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	/* Shouldn't gnome-vfs have a list of schemes it supports? */
 	if (g_str_has_prefix (url, "mms") != FALSE
 			|| g_str_has_prefix (url, "rtsp") != FALSE) {
+		DEBUG(g_print ("URL '%s' is MMS or RTSP, ignoring\n", url));
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
@@ -1746,8 +1836,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	}
 
 	mimetype = gnome_vfs_mime_type_from_name (url);
+	DEBUG(g_print ("_mime_type_from_name for '%s' returned '%s'\n", url, mimetype));
 	if (mimetype == NULL || strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mimetype) == 0) {
-		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
+		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
+		DEBUG(g_print ("_get_mime_type_with_data for '%s' returned '%s'\n", url, mimetype ? mimetype : "NULL"));
 	}
 
 	if (mimetype == NULL)
@@ -1761,9 +1853,9 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (parser->priv->recurse || parser->priv->recurse_level == 0) {
 		parser->priv->recurse_level++;
 
-		found = FALSE;
 		for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 			if (strcmp (special_types[i].mimetype, mimetype) == 0) {
+				DEBUG(g_print ("URL '%s' is special type '%s'\n", url, mimetype));
 				ret = (* special_types[i].func) (parser, url, data);
 				found = TRUE;
 				break;
@@ -1772,10 +1864,12 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 
 		for (i = 0; i < G_N_ELEMENTS(dual_types) && found == FALSE; i++) {
 			if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
+				DEBUG(g_print ("URL '%s' is dual type '%s'\n", url, mimetype));
 				if (data == NULL) {
-					mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
+					mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
 				}
 				ret = (* dual_types[i].func) (parser, url, data);
+				found = TRUE;
 				break;
 			}
 		}
@@ -1789,7 +1883,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		return ret;
 
 	super = gnome_vfs_get_supertype_from_mime_type (mimetype);
-	for (i = 0; i < G_N_ELEMENTS (ignore_types) && super != NULL; i++) {
+	for (i = 0; i < G_N_ELEMENTS (ignore_types) && super != NULL && found == FALSE; i++) {
 		if (gnome_vfs_mime_type_is_supertype (ignore_types[i].mimetype) != FALSE) {
 			if (strcmp (super, ignore_types[i].mimetype) == 0) {
 				g_free (super);
