@@ -96,6 +96,7 @@ struct Totem {
 	gboolean seek_lock;
 	gboolean seekable;
 	gboolean was_playing;
+	gboolean seek_in_progress;
 
 	/* Volume */
 	GtkWidget *volume;
@@ -150,6 +151,7 @@ struct Totem {
 	TotemRemote *remote;
 	BaconMessageConnection *conn;
 	TotemStates state;
+	gboolean cursor_shown;
 };
 
 GtkWidget *totem_volume_create (void);
